@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: mat_tools.cpp 2141 2014-06-06 13:57:12Z oconrad $
+ * Version $Id: mat_tools.cpp 2457 2015-03-23 11:27:57Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -101,6 +101,36 @@ double			SG_Get_Rounded(double Value, int Decimals)
 	return( ((int)(0.5 + d * Value)) / d );
 }
 
+//---------------------------------------------------------
+double			SG_Get_Rounded_To_SignificantFigures(double Value, int Decimals)
+{
+	if( Decimals <= 0 || Value == 0.0 )
+	{
+		return( (int)(0.5 + Value) );
+	}
+
+	Decimals	= (int)(-(ceil(log10(fabs(Value))) - Decimals));
+
+	if( Decimals > 0 )
+	{
+		double	d	= pow(10.0, Decimals);
+
+		return( Value < 0.0
+			? -((int)(0.5 - Value * d)) / d
+			:  ((int)(0.5 + Value * d)) / d
+		); 
+	}
+	else
+	{
+		double	d	= pow(10.0, -Decimals);
+
+		return( Value < 0.0
+			? -((int)(0.5 - Value / d)) * d
+			:  ((int)(0.5 + Value / d)) * d
+		);
+	}
+}
+
 
 ///////////////////////////////////////////////////////////
 //                                                       //
@@ -145,6 +175,43 @@ CSG_String		SG_Get_Double_asString(double Number, int Width, int Precision, bool
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+int SG_Compare_Int(const void *a, const void *b)
+{
+	if( *((int *)a) < *((int *)b) )
+		return( -1  );
+
+	if( *((int *)a) > *((int *)b) )
+		return(  1  );
+
+	return(  0  );
+}
+
+//---------------------------------------------------------
+int SG_Compare_Double(const void *a, const void *b)
+{
+	if( *((double *)a) < *((double *)b) )
+		return( -1  );
+
+	if( *((double *)a) > *((double *)b) )
+		return(  1  );
+
+	return(  0  );
+}
+
+//---------------------------------------------------------
+int SG_Compare_Char_Ptr(const void *a, const void *b)
+{
+	return( strcmp((const char *)a, (const char *)b) );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
@@ -242,6 +309,11 @@ CSG_Simple_Statistics::CSG_Simple_Statistics(double Mean, double StdDev, sLong C
 	Create(Mean, StdDev, Count);
 }
 
+CSG_Simple_Statistics::CSG_Simple_Statistics(const CSG_Vector &Values, bool bHoldValues)
+{
+	Create(Values, bHoldValues);
+}
+
 //---------------------------------------------------------
 bool CSG_Simple_Statistics::Create(bool bHoldValues)
 {
@@ -268,6 +340,7 @@ bool CSG_Simple_Statistics::Create(const CSG_Simple_Statistics &Statistics)
 	m_Variance		= Statistics.m_Variance;
 	m_StdDev		= Statistics.m_StdDev;
 
+	m_bSorted		= Statistics.m_bSorted;
 	m_Values		.Create(Statistics.m_Values);
 
 	return( true );
@@ -290,11 +363,27 @@ bool CSG_Simple_Statistics::Create(double Mean, double StdDev, sLong Count)
 	m_Maximum		= m_Mean + 1.5 * m_StdDev;
 	m_Range			= m_Maximum - m_Minimum;
 
+	m_bSorted		= false;
 	m_Values		.Destroy();
 
 	return( true );
 }
 
+bool CSG_Simple_Statistics::Create(const CSG_Vector &Values, bool bHoldValues)
+{
+	if( Create(bHoldValues) )
+	{
+		for(size_t i=0; i<Values.Get_Size(); i++)
+		{
+			Add_Value(Values[i]);
+		}
+
+		return( true );
+	}
+
+	return( false );
+}
+
 //---------------------------------------------------------
 void CSG_Simple_Statistics::Invalidate(void)
 {
@@ -312,6 +401,7 @@ void CSG_Simple_Statistics::Invalidate(void)
 	m_Variance		= 0.0;
 	m_StdDev		= 0.0;
 
+	m_bSorted		= false;
 	m_Values		.Destroy();
 }
 
@@ -331,7 +421,7 @@ void CSG_Simple_Statistics::Add(const CSG_Simple_Statistics &Statistics)
 	}
 
 	//--------------------------------------------------------
-	if( m_Values.Get_Size() == m_nValues && Statistics.m_Values.Get_Size() == Statistics.m_nValues && m_Values.Set_Array(m_nValues + Statistics.m_nValues) )
+	if( m_Values.Get_Size() == m_nValues && Statistics.m_Values.Get_Size() == Statistics.m_nValues && m_Values.Set_Array((size_t)(m_nValues + Statistics.m_nValues)) )
 	{
 		for(sLong i=0, j=m_nValues; i<Statistics.m_nValues; i++, j++)
 		{
@@ -355,6 +445,7 @@ void CSG_Simple_Statistics::Add(const CSG_Simple_Statistics &Statistics)
 		m_Maximum	= Statistics.m_Maximum;
 
 	m_bEvaluated	= false;
+	m_bSorted		= false;
 }
 
 //---------------------------------------------------------
@@ -384,6 +475,8 @@ void CSG_Simple_Statistics::Add_Value(double Value, double Weight)
 
 	if( m_Values.Get_Value_Size() > 0 && m_Values.Inc_Array() )
 	{
+		m_bSorted		= false;
+
 		((double *)m_Values.Get_Array())[m_nValues]	= Value;
 	}
 
@@ -404,6 +497,30 @@ void CSG_Simple_Statistics::_Evaluate(void)
 	}
 }
 
+//---------------------------------------------------------
+/**
+  * The quantile is expected to be given as percentage.
+  * A percentage of 50 returns the median. Remark:
+  * Quantile calculation is only possible, if statistics
+  * has been created with the bHoldValues option set to true.
+*/
+double CSG_Simple_Statistics::Get_Quantile(double Quantile)
+{
+	if( m_Values.Get_Size() > 0 )
+	{
+		if( !m_bSorted )
+		{
+			qsort(m_Values.Get_Array(), m_Values.Get_Size(), sizeof(double), SG_Compare_Double);
+
+			m_bSorted	= true;
+		}
+
+		return( Get_Value((sLong)(0.5 + (m_Values.Get_Size() - 1) * Quantile / 100.0)) );
+	}
+
+	return( m_Mean );
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -903,14 +1020,15 @@ bool CSG_Cluster_Analysis::Hill_Climbing(bool bInitialize, int nMaxIterations)
 //---------------------------------------------------------
 CSG_Classifier_Supervised::CSG_Classifier_Supervised(void)
 {
-	m_Statistics			= NULL;
-	m_nElements				= NULL;
 	m_nFeatures				= 0;
 
-	m_Distance_Threshold	= 0.0;
-	m_Probability_Threshold	= 0.0;
+	m_nClasses				= 0;
+	m_pClasses				= NULL;
+
+	m_Threshold_Distance	= 0.0;
+	m_Threshold_Angle		= 0.0;
+	m_Threshold_Probability	= 0.0;
 	m_Probability_Relative	= false;
-	m_Angle_Threshold		= 0.0;
 
 	for(int i=0; i<SG_CLASSIFY_SUPERVISED_WTA; i++)
 	{
@@ -941,41 +1059,380 @@ void CSG_Classifier_Supervised::Create(int nFeatures)
 //---------------------------------------------------------
 void CSG_Classifier_Supervised::Destroy(void)
 {
-	if( Get_Class_Count() > 0 )
+	if( m_nClasses > 0 )
 	{
-		for(int i=0; i<Get_Class_Count(); i++)
+		for(int i=0; i<m_nClasses; i++)
 		{
-			delete[](m_Statistics[i]);
+			delete(m_pClasses[i]);
 		}
 
-		SG_Free(m_Statistics);
-		SG_Free(m_nElements);
-
-		m_Statistics	= NULL;
-		m_nElements		= NULL;
-
-		m_BE_m	.Destroy();
-		m_BE_s	.Destroy();
-		m_SAM_l	.Destroy();
-		m_ML_s	.Destroy();
-		m_ML_a	.Destroy();
-		m_ML_b	.Destroy();
-
-		m_IDs	.Clear();
+		SG_FREE_SAFE(m_pClasses);
 	}
 
 	m_nFeatures	= 0;
+
+	m_Info.Clear();
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+void   CSG_Classifier_Supervised::Set_Threshold_Distance   (double Value)	{	m_Threshold_Distance	= Value;	}
+double CSG_Classifier_Supervised::Get_Threshold_Distance   (void)			{	return( m_Threshold_Distance );		}
+
+//---------------------------------------------------------
+void   CSG_Classifier_Supervised::Set_Threshold_Angle      (double Value)	{	m_Threshold_Angle		= Value;	}
+double CSG_Classifier_Supervised::Get_Threshold_Angle      (void)			{	return( m_Threshold_Angle );		}
+
+//---------------------------------------------------------
+void   CSG_Classifier_Supervised::Set_Threshold_Probability(double Value)	{	m_Threshold_Probability	= Value;	}
+double CSG_Classifier_Supervised::Get_Threshold_Probability(void)			{	return( m_Threshold_Probability );	}
+
+//---------------------------------------------------------
+void   CSG_Classifier_Supervised::Set_Probability_Relative (bool   Value)	{	m_Probability_Relative	= Value;	}
+bool   CSG_Classifier_Supervised::Get_Probability_Relative (void)			{	return( m_Probability_Relative );	}
+
+//---------------------------------------------------------
+void CSG_Classifier_Supervised::Set_WTA(int Method, bool bOn)
+{
+	if( Method >= 0 && Method < SG_CLASSIFY_SUPERVISED_WTA )
+	{
+		m_bWTA[Method]	= bOn;
+	}
+}
+
+bool CSG_Classifier_Supervised::Get_WTA(int Method)
+{
+	return( Method >= 0 && Method < SG_CLASSIFY_SUPERVISED_WTA ? m_bWTA[Method] : false );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+#include "saga_api.h"
+
+//---------------------------------------------------------
+bool CSG_Classifier_Supervised::Load(const CSG_String &File)
+{
+	int	nFeatures	= m_nFeatures;	Destroy();	m_nFeatures	= nFeatures;
+
+	//-----------------------------------------------------
+	CSG_MetaData	Data;
+
+	if( !Data.Load(File) || !Data.Cmp_Name("supervised_classifier") || SG_Compare_Version(Data.Get_Property("saga-version"), "2.1.4") < 0 )
+	{
+		return( false );
+	}
+
+	if( !Data("classes") || !Data("features") || !Data["features"]("count") || Data["features"]["count"].Get_Content().asInt() != m_nFeatures || m_nFeatures == 0 )
+	{
+		return( false );
+	}
+
+	if( Data["features"]("info") )
+	{
+		m_Info	= Data["features"]["info"].Get_Content();
+	}
+
+	//-----------------------------------------------------
+	CSG_MetaData	&Classes	= *Data.Get_Child("CLASSES");
+
+	for(int i=0; i<Classes.Get_Children_Count(); i++)
+	{
+		if( Classes[i].Cmp_Name("class") && Classes[i].Get_Child("id") )
+		{
+			bool	bAdd	= true;
+
+			CClass	*pClass	= new CClass(Classes[i]["id"].Get_Content());
+
+			if( !pClass->m_Cov .from_String(Classes[i]["cov" ].Get_Content()) || pClass->m_Cov .Get_NX() != m_nFeatures || !pClass->m_Cov.is_Square() )	{	bAdd	= false;	}
+			if( !pClass->m_Mean.from_String(Classes[i]["mean"].Get_Content()) || pClass->m_Mean.Get_N () != m_nFeatures )	{	bAdd	= false;	}
+			if( !pClass->m_Min .from_String(Classes[i]["min" ].Get_Content()) || pClass->m_Min .Get_N () != m_nFeatures )	{	bAdd	= false;	}
+			if( !pClass->m_Max .from_String(Classes[i]["max" ].Get_Content()) || pClass->m_Max .Get_N () != m_nFeatures )	{	bAdd	= false;	}
+
+			//---------------------------------------------
+			if( !bAdd )
+			{
+				delete(pClass);
+			}
+			else
+			{
+				m_pClasses	= (CClass **)SG_Realloc(m_pClasses, (m_nClasses + 1) * sizeof(CClass *));
+				m_pClasses[m_nClasses++]	= pClass;
+
+				pClass->m_Cov_Det	= pClass->m_Cov.Get_Determinant();
+				pClass->m_Cov_Inv	= pClass->m_Cov.Get_Inverse();
+
+				pClass->m_Mean_Spectral	= CSG_Simple_Statistics(pClass->m_Mean).Get_Mean();
+			}
+		}
+	}
+
+	return( m_nClasses > 0 );
 }
 
 //---------------------------------------------------------
-void CSG_Classifier_Supervised::Del_Element_Count(void)
+bool CSG_Classifier_Supervised::Save(const CSG_String &File, const SG_Char *Feature_Info)
 {
-	for(int i=0; i<Get_Class_Count(); i++)
+	if( m_nFeatures < 1 || m_nClasses < 1 || File.is_Empty() )
 	{
-		m_nElements[i]	= 0;
+		return( false );
 	}
+
+	CSG_MetaData	Data;
+
+	Data.Set_Name    ("supervised_classifier");
+	Data.Add_Property("saga-version", SAGA_VERSION);
+
+	CSG_MetaData	&Features	= *Data.Add_Child("features");
+	
+	Features.Add_Child("count", m_nFeatures);
+
+	if( Feature_Info && *Feature_Info )
+	{
+		Features.Add_Child("info", Feature_Info);
+	}
+
+	CSG_MetaData	&Classes	= *Data.Add_Child("classes");
+	
+	Classes.Add_Property("count", m_nClasses);
+
+	for(int i=0; i<m_nClasses; i++)
+	{
+		CSG_MetaData	&Class	= *Classes.Add_Child("class");
+
+		CClass	*pClass	= m_pClasses[i];
+
+		Class.Add_Child("id"  , pClass->m_ID              );
+		Class.Add_Child("mean", pClass->m_Mean.to_String());
+		Class.Add_Child("min" , pClass->m_Min .to_String());
+		Class.Add_Child("max" , pClass->m_Max .to_String());
+		Class.Add_Child("cov" , pClass->m_Cov .to_String());
+	}
+
+	return( Data.Save(File) );
 }
 
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+CSG_String CSG_Classifier_Supervised::Print(void)
+{
+	CSG_String	s;
+
+	if( m_nFeatures > 0 && m_nClasses > 0 )
+	{
+		s	+= "\n";
+
+		for(int iClass=0; iClass<m_nClasses; iClass++)
+		{
+			CClass	*pClass	= m_pClasses[iClass];
+
+			s	+= "\n____\n" + pClass->m_ID + "\nFeature\tMean\tMin\tMax\tStdDev";
+
+			for(int i=0; i<m_nFeatures; i++)
+			{
+				s	+= CSG_String::Format("\n%3d.", i + 1);
+				s	+= "\t" + SG_Get_String(pClass->m_Mean[i]);
+				s	+= "\t" + SG_Get_String(pClass->m_Min [i]);
+				s	+= "\t" + SG_Get_String(pClass->m_Max [i]);
+				s	+= "\t" + SG_Get_String(sqrt(pClass->m_Cov[i][i]));
+			}
+
+			s	+= "\n";
+		}
+	}
+
+	return( s );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_Classifier_Supervised::Add_Class(const CSG_String &Class_ID, const CSG_Vector &Mean, const CSG_Vector &Min, const CSG_Vector &Max, const CSG_Matrix &Cov)
+{
+	if( m_nFeatures < 1 || Mean.Get_N() != m_nFeatures || Min.Get_N() != m_nFeatures || Max.Get_N() != m_nFeatures || Cov.Get_NCols() != m_nFeatures || Cov.Get_NRows() != m_nFeatures )
+	{
+		return( false );
+	}
+
+	CClass	*pClass, **pClasses	= (CClass **)SG_Realloc(m_pClasses, (m_nClasses + 1) * sizeof(CClass *));
+
+	if( pClasses )
+	{
+		m_pClasses	= pClasses;
+
+		m_pClasses[m_nClasses++]	= pClass	= new CClass(Class_ID);
+
+		pClass->m_ID	= Class_ID;
+
+		pClass->m_Mean	= Mean;
+		pClass->m_Min	= Min;
+		pClass->m_Max	= Max;
+		pClass->m_Cov	= Cov;
+
+		pClass->m_Cov_Inv	= Cov.Get_Inverse();
+		pClass->m_Cov_Det	= Cov.Get_Determinant();
+
+		pClass->m_Mean_Spectral	= CSG_Simple_Statistics(Mean).Get_Mean();
+
+		return( true );
+	}
+
+	return( false );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_Classifier_Supervised::Train_Clr_Samples(void)
+{
+	for(int i=0; i<m_nClasses; i++)
+	{
+		m_pClasses[i]->m_Samples.Destroy();
+	}
+
+	return( true );
+}
+
+//---------------------------------------------------------
+bool CSG_Classifier_Supervised::Train_Add_Sample(const CSG_String &Class_ID, const CSG_Vector &Features)
+{
+	if( m_nFeatures > 0 && m_nFeatures == Features.Get_N() )
+	{
+		int	iClass	= Get_Class(Class_ID);
+
+		if( iClass < 0 )
+		{
+			CClass	**pClasses	= (CClass **)SG_Realloc(m_pClasses, (m_nClasses + 1) * sizeof(CClass *));
+
+			if( pClasses )
+			{
+				m_pClasses	= pClasses;
+
+				m_pClasses[iClass = m_nClasses++]	= new CClass(Class_ID);
+			}
+		}
+
+		if( iClass >= 0 )
+		{
+			return( m_pClasses[iClass]->m_Samples.Add_Row(Features) );
+		}
+	}
+
+	return( false );
+}
+
+//---------------------------------------------------------
+bool CSG_Classifier_Supervised::Train(bool bClear_Samples)
+{
+	if( m_nFeatures < 1 || m_nClasses < 1 )
+	{
+		return( false );
+	}
+
+	for(int iClass=0; iClass<m_nClasses; iClass++)
+	{
+		if( !m_pClasses[iClass]->Train() )
+		{
+			return( false );
+		}
+	}
+
+	if( bClear_Samples )
+	{
+		Train_Clr_Samples();
+	}
+
+	return( true );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_Classifier_Supervised::CClass::Train(void)
+{
+	if( m_Samples.Get_NCols() < 1 || m_Samples.Get_NRows() < 1 )
+	{
+		return( false );
+	}
+
+	int	iFeature;
+
+	//-----------------------------------------------------
+	m_Mean.Create(m_Samples.Get_NCols());
+	m_Min .Create(m_Samples.Get_NCols());
+	m_Max .Create(m_Samples.Get_NCols());
+
+	for(iFeature=0; iFeature<m_Samples.Get_NCols(); iFeature++)
+	{
+		CSG_Simple_Statistics	s;
+
+		for(int iSample=0; iSample<m_Samples.Get_NRows(); iSample++)
+		{
+			s	+= m_Samples[iSample][iFeature];
+		}
+
+		m_Mean[iFeature]	= s.Get_Mean   ();
+		m_Min [iFeature]	= s.Get_Minimum();
+		m_Max [iFeature]	= s.Get_Maximum();
+	}
+
+	//-----------------------------------------------------
+	m_Cov.Create(m_Samples.Get_NCols(), m_Samples.Get_NCols());
+
+	for(iFeature=0; iFeature<m_Samples.Get_NCols(); iFeature++)
+	{
+		for(int jFeature=iFeature; jFeature<m_Samples.Get_NCols(); jFeature++)
+		{
+			double	cov	= 0.0;
+
+			for(int iSample=0; iSample<m_Samples.Get_NRows(); iSample++)
+			{
+				cov	+= (m_Samples[iSample][iFeature] - m_Mean[iFeature]) * (m_Samples[iSample][jFeature] - m_Mean[jFeature]); 
+			}
+
+			if( m_Samples.Get_NRows() > 1 )
+			{
+				cov	/= m_Samples.Get_NRows() - 1;
+			}
+
+			m_Cov[iFeature][jFeature]	= m_Cov[jFeature][iFeature]	= cov;
+		}
+	}
+
+	m_Cov_Inv	= m_Cov.Get_Inverse    ();
+	m_Cov_Det	= m_Cov.Get_Determinant();
+
+	m_Mean_Spectral	= CSG_Simple_Statistics(m_Mean).Get_Mean();
+
+	//-----------------------------------------------------
+	return( true );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
 //---------------------------------------------------------
 int CSG_Classifier_Supervised::Get_Class(const CSG_String &Class_ID)
 {
@@ -994,83 +1451,29 @@ int CSG_Classifier_Supervised::Get_Class(const CSG_String &Class_ID)
 }
 
 //---------------------------------------------------------
-CSG_Simple_Statistics * CSG_Classifier_Supervised::Get_Statistics(const CSG_String &Class_ID)
+bool CSG_Classifier_Supervised::Get_Class(const CSG_Vector &Features, int &Class, double &Quality, int Method)
 {
-	if( m_nFeatures > 0 )
+	Class	= -1;
+	Quality	= 0.0;
+
+	if( Get_Feature_Count() == Features.Get_N() )
 	{
-		int		iClass	= Get_Class(Class_ID);
-
-		if( iClass < 0 )
+		switch( Method )
 		{
-			iClass					 = Get_Class_Count();
-
-			m_IDs					+= Class_ID;
-
-			m_nElements				 = (int *)SG_Realloc(m_nElements, m_IDs.Get_Count() * sizeof(int));
-			m_nElements[iClass]		 = 0;
-
-			m_Statistics			 = (CSG_Simple_Statistics **)SG_Realloc(m_Statistics, m_IDs.Get_Count() * sizeof(CSG_Simple_Statistics *));
-			m_Statistics[iClass]	 = new CSG_Simple_Statistics[m_nFeatures];
+		case SG_CLASSIFY_SUPERVISED_BinaryEncoding   :	_Get_Binary_Encoding       (Features, Class, Quality);	break;
+		case SG_CLASSIFY_SUPERVISED_ParallelEpiped   :	_Get_Parallel_Epiped       (Features, Class, Quality);	break;
+		case SG_CLASSIFY_SUPERVISED_MinimumDistance  :	_Get_Minimum_Distance      (Features, Class, Quality);	break;
+		case SG_CLASSIFY_SUPERVISED_Mahalonobis      :	_Get_Mahalanobis_Distance  (Features, Class, Quality);	break;
+		case SG_CLASSIFY_SUPERVISED_MaximumLikelihood:	_Get_Maximum_Likelihood    (Features, Class, Quality);	break;
+		case SG_CLASSIFY_SUPERVISED_SAM              :	_Get_Spectral_Angle_Mapping(Features, Class, Quality);	break;
+		case SG_CLASSIFY_SUPERVISED_SID              :	_Get_Spectral_Divergence   (Features, Class, Quality);	break;
+		case SG_CLASSIFY_SUPERVISED_WTA              :	_Get_Winner_Takes_All      (Features, Class, Quality);	break;
 		}
 
-		return( m_Statistics[iClass] );
+		return( Class >= 0 );
 	}
 
-	return( NULL );
-}
-
-
-///////////////////////////////////////////////////////////
-//														 //
-///////////////////////////////////////////////////////////
-
-//---------------------------------------------------------
-void CSG_Classifier_Supervised::_Update(void)
-{
-	if( m_SAM_l.Get_N() != Get_Class_Count() )
-	{
-		int		iClass, iFeature;
-
-		m_BE_s	.Create(Get_Feature_Count(), Get_Class_Count());
-		m_BE_m	.Create(Get_Class_Count());
-		m_SAM_l	.Create(Get_Class_Count());
-		m_ML_s	.Create(Get_Class_Count());
-		m_ML_a	.Create(Get_Feature_Count(), Get_Class_Count());
-		m_ML_b	.Create(Get_Feature_Count(), Get_Class_Count());
-
-		for(iClass=0; iClass<Get_Class_Count(); iClass++)
-		{
-			CSG_Simple_Statistics	*Statistic	= m_Statistics[iClass];
-
-			double	m	= 0.0;
-			double	l	= 0.0;
-			double	s	= 1.0;
-
-			for(iFeature=0; iFeature<Get_Feature_Count(); iFeature++)
-			{
-				m	+= Statistic[iFeature].Get_Mean();
-				l	+= SG_Get_Square(Statistic[iFeature].Get_Mean());
-				s	*= Statistic[iFeature].Get_Variance();
-
-				m_ML_a[iClass][iFeature]	=  1.0 / sqrt(Statistic[iFeature].Get_Variance() * 2.0 * M_PI);
-				m_ML_b[iClass][iFeature]	= -1.0 /     (Statistic[iFeature].Get_Variance() * 2.0);
-			}
-
-			m_BE_m  [iClass]	= m / Get_Feature_Count();
-			m_SAM_l	[iClass]	= sqrt(l);
-			m_ML_s	[iClass]	= 1.0 / (pow(2.0 * M_PI, Get_Feature_Count() / 2.0) * sqrt(s));
-		}
-
-		for(iClass=0; iClass<Get_Class_Count(); iClass++)
-		{
-			CSG_Simple_Statistics	*Statistic	= m_Statistics[iClass];
-
-			for(iFeature=0; iFeature<Get_Feature_Count(); iFeature++)
-			{
-				m_BE_s[iClass][iFeature]	= Statistic[iFeature].Get_Mean() < m_BE_m[iClass] ? 0.0 : 1.0;
-			}
-		}
-	}
+	return( false );
 }
 
 
@@ -1083,15 +1486,15 @@ CSG_String CSG_Classifier_Supervised::Get_Name_of_Method(int Method)
 {
 	switch( Method )
 	{
-	case SG_CLASSIFY_SUPERVISED_BinaryEncoding:		return( _TL("Binary Encoding") );
-	case SG_CLASSIFY_SUPERVISED_ParallelEpiped:		return( _TL("Parallelepiped") );
-	case SG_CLASSIFY_SUPERVISED_MinimumDistance:	return( _TL("Minimum Distance") );
-	case SG_CLASSIFY_SUPERVISED_Mahalonobis:		return( _TL("Mahalanobis Distance") );
+	case SG_CLASSIFY_SUPERVISED_BinaryEncoding   :	return( _TL("Binary Encoding") );
+	case SG_CLASSIFY_SUPERVISED_ParallelEpiped   :	return( _TL("Parallelepiped") );
+	case SG_CLASSIFY_SUPERVISED_MinimumDistance  :	return( _TL("Minimum Distance") );
+	case SG_CLASSIFY_SUPERVISED_Mahalonobis      :	return( _TL("Mahalanobis Distance") );
 	case SG_CLASSIFY_SUPERVISED_MaximumLikelihood:	return( _TL("Maximum Likelihood") );
-	case SG_CLASSIFY_SUPERVISED_SAM:				return( _TL("Spectral Angle Mapping") );
-	case SG_CLASSIFY_SUPERVISED_SID:				return( _TL("Spectral Information Divergence") );
-	case SG_CLASSIFY_SUPERVISED_SVM:				return( _TL("Support Vector Machine") );
-	case SG_CLASSIFY_SUPERVISED_WTA:				return( _TL("Winner Takes All") );
+	case SG_CLASSIFY_SUPERVISED_SAM              :	return( _TL("Spectral Angle Mapping") );
+	case SG_CLASSIFY_SUPERVISED_SID              :	return( _TL("Spectral Information Divergence") );
+	case SG_CLASSIFY_SUPERVISED_SVM              :	return( _TL("Support Vector Machine") );
+	case SG_CLASSIFY_SUPERVISED_WTA              :	return( _TL("Winner Takes All") );
 	}
 
 	return( SG_T("") );
@@ -1102,74 +1505,59 @@ CSG_String CSG_Classifier_Supervised::Get_Name_of_Quality(int Method)
 {
 	switch( Method )
 	{
-	case SG_CLASSIFY_SUPERVISED_BinaryEncoding:		return( _TL("Difference") );
-	case SG_CLASSIFY_SUPERVISED_ParallelEpiped:		return( _TL("Memberships") );
-	case SG_CLASSIFY_SUPERVISED_MinimumDistance:	return( _TL("Distance") );
-	case SG_CLASSIFY_SUPERVISED_Mahalonobis:		return( _TL("Distance") );
+	case SG_CLASSIFY_SUPERVISED_BinaryEncoding   :	return( _TL("Difference") );
+	case SG_CLASSIFY_SUPERVISED_ParallelEpiped   :	return( _TL("Memberships") );
+	case SG_CLASSIFY_SUPERVISED_MinimumDistance  :	return( _TL("Distance") );
+	case SG_CLASSIFY_SUPERVISED_Mahalonobis      :	return( _TL("Distance") );
 	case SG_CLASSIFY_SUPERVISED_MaximumLikelihood:	return( _TL("Proximity") );
-	case SG_CLASSIFY_SUPERVISED_SAM:				return( _TL("Angle") );
-	case SG_CLASSIFY_SUPERVISED_SID:				return( _TL("Divergence") );
-	case SG_CLASSIFY_SUPERVISED_SVM:				return( _TL("") );
-	case SG_CLASSIFY_SUPERVISED_WTA:				return( _TL("Votes") );
+	case SG_CLASSIFY_SUPERVISED_SAM              :	return( _TL("Angle") );
+	case SG_CLASSIFY_SUPERVISED_SID              :	return( _TL("Divergence") );
+	case SG_CLASSIFY_SUPERVISED_SVM              :	return( _TL("") );
+	case SG_CLASSIFY_SUPERVISED_WTA              :	return( _TL("Votes") );
 	}
 
 	return( SG_T("") );
 }
 
-//---------------------------------------------------------
-bool CSG_Classifier_Supervised::Get_Class(const CSG_Vector &Features, int &Class, double &Quality, int Method)
-{
-	Class	= -1;
-	Quality	= 0.0;
-
-	if( Get_Feature_Count() == Features.Get_N() )
-	{
-		_Update();
-
-		switch( Method )
-		{
-		case SG_CLASSIFY_SUPERVISED_BinaryEncoding:		_Get_Binary_Encoding       (Features, Class, Quality);	break;
-		case SG_CLASSIFY_SUPERVISED_ParallelEpiped:		_Get_Parallel_Epiped       (Features, Class, Quality);	break;
-		case SG_CLASSIFY_SUPERVISED_MinimumDistance:	_Get_Minimum_Distance      (Features, Class, Quality);	break;
-		case SG_CLASSIFY_SUPERVISED_Mahalonobis:		_Get_Mahalanobis_Distance  (Features, Class, Quality);	break;
-		case SG_CLASSIFY_SUPERVISED_MaximumLikelihood:	_Get_Maximum_Likelihood    (Features, Class, Quality);	break;
-		case SG_CLASSIFY_SUPERVISED_SAM:				_Get_Spectral_Angle_Mapping(Features, Class, Quality);	break;
-		case SG_CLASSIFY_SUPERVISED_SID:				_Get_Spectral_Divergence   (Features, Class, Quality);	break;
-		case SG_CLASSIFY_SUPERVISED_WTA:				_Get_Winner_Takes_All      (Features, Class, Quality);	break;
-		}
-
-		if( Class >= 0 )
-		{
-			m_nElements[Class]++;
-
-			return( true );
-		}
-	}
-
-	return( false );
-}
-
 
 ///////////////////////////////////////////////////////////
 //														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
+// Mazer, A. S., Martin, M., Lee, M., and Solomon, J. E. (1988):
+// Image Processing Software for Imaging Spectrometry Analysis.
+// Remote Sensing of Environment, v. 24, no. 1, p. 201-210.
+//
 void CSG_Classifier_Supervised::_Get_Binary_Encoding(const CSG_Vector &Features, int &Class, double &Quality)
 {
 	for(int iClass=0; iClass<Get_Class_Count(); iClass++)
 	{
+		CClass	*pClass	= m_pClasses[iClass];
+
+		double	Mean_Spectral	= CSG_Simple_Statistics(Features).Get_Mean();
+
 		int		d	= 0;
 
 		for(int iFeature=0; iFeature<Get_Feature_Count(); iFeature++)
 		{
-			if(	(Features(iFeature) < m_BE_m[iClass]) != (m_BE_s[iClass][iFeature] != 0.0) )
+			d	+= (Features(iFeature) < Mean_Spectral) == (pClass->m_Mean[iFeature] < pClass->m_Mean_Spectral) ? 0 : 1;
+
+			if( iFeature == 0 )	// spectral slopes
 			{
-				d	++;
+				d	+= (Features[iFeature    ] < Features[iFeature + 1]) == (pClass->m_Mean[iFeature    ] < pClass->m_Mean[iFeature + 1]) ? 0 : 1;
+			}
+			else if( iFeature == Get_Feature_Count() - 1 )
+			{
+				d	+= (Features[iFeature - 1] < Features[iFeature    ]) == (pClass->m_Mean[iFeature - 1] < pClass->m_Mean[iFeature    ]) ? 0 : 1;
+			}
+			else
+			{
+				d	+= (Features[iFeature - 1] < Features[iFeature + 1]) == (pClass->m_Mean[iFeature - 1] < pClass->m_Mean[iFeature + 1]) ? 0 : 1;
 			}
 		}
 
-		if( Class < 0 || Quality < d )
+		if( Class < 0 || Quality > d )	// find the minimum 'Hamming' distance
 		{
 			Quality	= d;
 			Class	= iClass;
@@ -1182,17 +1570,13 @@ void CSG_Classifier_Supervised::_Get_Parallel_Epiped(const CSG_Vector &Features,
 {
 	for(int iClass=0; iClass<Get_Class_Count(); iClass++)
 	{
+		CClass	*pClass	= m_pClasses[iClass];
+
 		bool	bMember	= true;
 
 		for(int iFeature=0; bMember && iFeature<Get_Feature_Count(); iFeature++)
 		{
-			double	d	= Features(iFeature);
-
-			if(	d < m_Statistics[iClass][iFeature].Get_Minimum()
-			||	d > m_Statistics[iClass][iFeature].Get_Maximum() )
-			{
-				bMember	= false;
-			}
+			bMember	= pClass->m_Min[iFeature] <= Features[iFeature] && Features[iFeature] <= pClass->m_Max[iFeature];
 		}
 
 		if( bMember )
@@ -1208,23 +1592,18 @@ void CSG_Classifier_Supervised::_Get_Minimum_Distance(const CSG_Vector &Features
 {
 	for(int iClass=0; iClass<Get_Class_Count(); iClass++)
 	{
-		double	d	= 0.0;
+		CClass	*pClass	= m_pClasses[iClass];
 
-		for(int iFeature=0; iFeature<Get_Feature_Count(); iFeature++)
-		{
-			d	+= SG_Get_Square(Features(iFeature) - m_Statistics[iClass][iFeature].Get_Mean());
-		}
+		double	Distance	= (Features - pClass->m_Mean).Get_Length();
 
-		if( Class < 0 || Quality > d )
+		if( Class < 0 || Quality > Distance )
 		{
-			Quality	= d;
+			Quality	= Distance;
 			Class	= iClass;
 		}
 	}
 
-	Quality	= sqrt(Quality);
-
-	if( m_Distance_Threshold > 0.0 && Quality > m_Distance_Threshold )
+	if( m_Threshold_Distance > 0.0 && Quality > m_Threshold_Distance )
 	{
 		Class	= -1;
 	}
@@ -1235,23 +1614,20 @@ void CSG_Classifier_Supervised::_Get_Mahalanobis_Distance(const CSG_Vector &Feat
 {
 	for(int iClass=0; iClass<Get_Class_Count(); iClass++)
 	{
-		double	d	= 0.0;
+		CClass	*pClass	= m_pClasses[iClass];
 
-		for(int iFeature=0; iFeature<Get_Feature_Count(); iFeature++)
-		{
-			d	+= SG_Get_Square((Features(iFeature) - m_Statistics[iClass][iFeature].Get_Mean()) / m_Statistics[iClass][iFeature].Get_StdDev());
-		}
+		CSG_Vector	D	= Features - pClass->m_Mean;
 
-		if( Class < 0 || Quality > d )
+		double	Distance	= D * (pClass->m_Cov_Inv * D);
+
+		if( Class < 0 || Quality > Distance )
 		{
-			Quality	= d;
+			Quality	= Distance;
 			Class	= iClass;
 		}
 	}
 
-	Quality	= sqrt(Quality);
-
-	if( m_Distance_Threshold > 0.0 && Quality > m_Distance_Threshold )
+	if( m_Threshold_Distance > 0.0 && Quality > m_Threshold_Distance )
 	{
 		Class	= -1;
 	}
@@ -1264,38 +1640,35 @@ void CSG_Classifier_Supervised::_Get_Maximum_Likelihood(const CSG_Vector &Featur
 
 	for(int iClass=0; iClass<Get_Class_Count(); iClass++)
 	{
-		double	d	= 1.0;
+		CClass	*pClass	= m_pClasses[iClass];
 
-		for(int iFeature=0; iFeature<Get_Feature_Count(); iFeature++)
+		CSG_Vector	D	= Features - pClass->m_Mean;
+
+		double	Distance	= D * (pClass->m_Cov_Inv * D);
+
+		double	Probability	= pow(2.0 * M_PI, -0.5 * m_nFeatures) * pow(pClass->m_Cov_Det, -0.5) * exp(-0.5 * Distance);
+	//	double	Probability	= -log(pClass->m_Cov_Det) - Distance;
+
+		dSum	+= Probability;
+
+		if( Class < 0 || Quality < Probability )
 		{
-			d	*= m_ML_a[iClass][iFeature] * exp(m_ML_b[iClass][iFeature] * SG_Get_Square(Features(iFeature) - m_Statistics[iClass][iFeature].Get_Mean()));
-		}
-
-		dSum	+= (d	= pow(d, 1.0 / Get_Feature_Count()));
-
-		/*
-		double	d	= 0.0;
-
-		for(int iFeature=0; iFeature<Get_Feature_Count(); iFeature++)
-		{
-			d	+= SG_Get_Square((Features(iFeature) - m_Statistics[iClass][iFeature].Get_Mean()) / m_Statistics[iClass][iFeature].Get_StdDev());
-		}
-
-		dSum	+= (d	= m_ML_s[iClass] * exp(-0.5 * d));
-		*/
-
-		if( Quality < d )
-		{
-			Quality	= d;
+			Quality	= Probability;
 			Class	= iClass;
 		}
 	}
 
-	Quality	= m_Probability_Relative ? 100.0 * Quality / dSum : 100.0 * Quality;
-
-	if( m_Probability_Threshold > 0.0 && Quality < m_Probability_Threshold )
+	if( Class >= 0 )
 	{
-		Class	= -1;
+		if( m_Probability_Relative )
+		{
+			Quality	= 100.0 * Quality / dSum;
+		}
+
+		if( m_Threshold_Probability > 0.0 && Quality < m_Threshold_Probability )
+		{
+			Class	= -1;
+		}
 	}
 }
 
@@ -1304,29 +1677,20 @@ void CSG_Classifier_Supervised::_Get_Spectral_Angle_Mapping(const CSG_Vector &Fe
 {
 	for(int iClass=0; iClass<Get_Class_Count(); iClass++)
 	{
-		double	d	= 0.0;
-		double	e	= 0.0;
+		CClass	*pClass	= m_pClasses[iClass];
 
-		for(int iFeature=0; iFeature<Get_Feature_Count(); iFeature++)
+		double	Angle	= Features.Get_Angle(pClass->m_Mean);
+
+		if( Class < 0 || Quality > Angle )
 		{
-			double	v	= Features(iFeature);
-
-			d	+= v * m_Statistics[iClass][iFeature].Get_Mean();
-			e	+= v*v;
-		}
-
-		d	= acos(d / (sqrt(e) * m_SAM_l[iClass]));
-
-		if( Class < 0 || Quality > d )
-		{
-			Quality	= d;
+			Quality	= Angle;
 			Class	= iClass;
 		}
 	}
 
 	Quality	*= M_RAD_TO_DEG;
 
-	if( m_Angle_Threshold > 0.0 && Quality > m_Angle_Threshold )
+	if( m_Threshold_Angle > 0.0 && Quality > m_Threshold_Angle )
 	{
 		Class	= -1;
 	}
