@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: parameter.cpp 2256 2014-09-26 15:08:53Z oconrad $
+ * Version $Id: parameter.cpp 2415 2015-02-18 14:27:34Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -901,10 +901,18 @@ bool CSG_Parameters_Grid_Target::On_Parameters_Enable(CSG_Parameters *pParameter
   * Initializes the grid system from Extent and number of rows.
   * If bFitToCells is true, the extent is deflated by half a cell size,
   * so automatically adjusting the extent based on grid nodes to
-  * to an extent based on the grid cells.
+  * an extent based on the grid cells. If Rounding is greater than
+  * zero it specifies the number of significant figures to which
+  * the cell size is rounded and also adjuts the extent coordinates
+  * to be a multiple of cell size.
 */
-bool CSG_Parameters_Grid_Target::Set_User_Defined(CSG_Parameters *pParameters, const TSG_Rect &Extent, int Rows, bool bFitToCells)
+bool CSG_Parameters_Grid_Target::Set_User_Defined(CSG_Parameters *pParameters, const TSG_Rect &Extent, int Rows, bool bFitToCells, int Rounding)
 {
+	if( !SG_UI_Get_Window_Main() )	// no cancel button, so set parameters directly
+	{
+		pParameters	= m_pParameters;
+	}
+
 	if( !m_pParameters || !pParameters || m_pParameters->Get_Identifier().Cmp(pParameters->Get_Identifier()) )
 	{
 		return( false );
@@ -938,18 +946,34 @@ bool CSG_Parameters_Grid_Target::Set_User_Defined(CSG_Parameters *pParameters, c
 	//-----------------------------------------------------
 	m_bFitToCells	= bFitToCells;
 
-	double	Size	= r.Get_YRange() / Rows;
+	double	Size	= r.Get_YRange() / ((m_bFitToCells ? 1 : 0) + Rows);
 
+	if( Rounding > 0 )
+	{
+		Size	= SG_Get_Rounded_To_SignificantFigures(Size, Rounding);
+
+		r.m_rect.xMin	= Size * floor(r.m_rect.xMin / Size);
+		r.m_rect.yMin	= Size * floor(r.m_rect.yMin / Size);
+		r.m_rect.yMax	= Size * ceil (r.m_rect.yMax / Size);
+	}
+
+	//-----------------------------------------------------
 	int		Cols	= (bFitToCells ? 0 : 1) + (int)(r.Get_XRange() / Size);
 
-	pParameters->Get_Parameter(m_Prefix + "USER_XMIN")->Set_Value(r.Get_XMin());
-	pParameters->Get_Parameter(m_Prefix + "USER_XMAX")->Set_Value(r.Get_XMax());
-	pParameters->Get_Parameter(m_Prefix + "USER_YMIN")->Set_Value(r.Get_YMin());
-	pParameters->Get_Parameter(m_Prefix + "USER_YMAX")->Set_Value(r.Get_YMax());
-	pParameters->Get_Parameter(m_Prefix + "USER_SIZE")->Set_Value(Size);
-	pParameters->Get_Parameter(m_Prefix + "USER_COLS")->Set_Value(Cols);
-	pParameters->Get_Parameter(m_Prefix + "USER_ROWS")->Set_Value(Rows);
-	pParameters->Get_Parameter(m_Prefix + "USER_FITS")->Set_Value(m_bFitToCells);
+	r.m_rect.xMax	= r.Get_XMin() + Cols * Size;
+
+	bool	bCallback	= pParameters->Set_Callback(false);
+
+	pParameters->Set_Parameter(m_Prefix + "USER_XMIN", r.Get_XMin() );
+	pParameters->Set_Parameter(m_Prefix + "USER_XMAX", r.Get_XMax() );
+	pParameters->Set_Parameter(m_Prefix + "USER_YMIN", r.Get_YMin() );
+	pParameters->Set_Parameter(m_Prefix + "USER_YMAX", r.Get_YMax() );
+	pParameters->Set_Parameter(m_Prefix + "USER_SIZE", Size         );
+	pParameters->Set_Parameter(m_Prefix + "USER_COLS", Cols         );
+	pParameters->Set_Parameter(m_Prefix + "USER_ROWS", Rows         );
+	pParameters->Set_Parameter(m_Prefix + "USER_FITS", m_bFitToCells);
+
+	pParameters->Set_Callback(bCallback);
 
 	return( true );
 }
@@ -970,7 +994,63 @@ bool CSG_Parameters_Grid_Target::Set_User_Defined(CSG_Parameters *pParameters, d
 		Extent.m_rect.yMax	+= Size;
 	}
 
-	return( Set_User_Defined(pParameters, Extent, ny, bFitToCells) );
+	return( Set_User_Defined(pParameters, Extent, ny, bFitToCells, 0) );
+}
+
+//---------------------------------------------------------
+bool CSG_Parameters_Grid_Target::Set_User_Defined(CSG_Parameters *pParameters, CSG_Shapes *pPoints, int Scale, bool bFitToCells, int Rounding)
+{
+	if( !pPoints || pPoints->Get_Count() <= 0 || pPoints->Get_Extent().Get_Area() <= 0.0 )
+	{
+		return( false );
+	}
+
+	double	d	= sqrt(pPoints->Get_Extent().Get_Area() / pPoints->Get_Count());	// edge length of a square given as average area per point (cell size)
+
+	int	Rows	= (int)(0.5 + pPoints->Get_Extent().Get_YRange() / d);	if( Scale > 1 ) Rows *= Scale;
+
+	return( Set_User_Defined(pParameters, pPoints->Get_Extent(), Rows, bFitToCells, Rounding) );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_Parameters_Grid_Target::Cmd_Update(const TSG_Rect &Extent)
+{
+	if( m_pParameters && !SG_UI_Get_Window_Main() )	// tool is run by saga_cmd ...
+	{
+		if( m_pParameters->Get_Manager() == &SG_Get_Data_Manager() )	// ... but tool is not run as part of a tool chain
+		{
+			Set_User_Defined(m_pParameters, Extent);
+
+			SG_UI_Dlg_Parameters(m_pParameters, m_pParameters->Get_Name());
+
+			return( true );
+		}
+	}
+
+	return( false );
+}
+
+//---------------------------------------------------------
+bool CSG_Parameters_Grid_Target::Cmd_Update(CSG_Shapes *pPoints)
+{
+	if( m_pParameters && !SG_UI_Get_Window_Main() )	// tool is run by saga_cmd ...
+	{
+		if( m_pParameters->Get_Manager() == &SG_Get_Data_Manager() )	// ... but tool is not run as part of a tool chain
+		{
+			Set_User_Defined(m_pParameters, pPoints);
+
+			SG_UI_Dlg_Parameters(m_pParameters, m_pParameters->Get_Name());
+
+			return( true );
+		}
+	}
+
+	return( false );
 }
 
 
