@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: res_commands.cpp 2291 2014-10-16 13:46:55Z oconrad $
+ * Version $Id: res_commands.cpp 2527 2015-06-30 15:38:35Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -118,6 +118,7 @@ wxString CMD_Get_Name(int Cmd_ID)
 
 	case ID_CMD_INFO_CLEAR:				return( _TL("Clear") );
 	case ID_CMD_INFO_COPY:				return( _TL("Copy") );
+	case ID_CMD_INFO_SELECTALL:			return( _TL("Select All") );
 
 	case ID_CMD_DATASOURCE_REFRESH:		return( _TL("Refresh") );
 
@@ -134,8 +135,8 @@ wxString CMD_Get_Name(int Cmd_ID)
 
 	case ID_CMD_MODULES_OPEN:			return( _TL("Load Tool Library") );
 	case ID_CMD_MODULES_SEARCH:			return( _TL("Find and Run Tool") );
-	case ID_CMD_MODULES_SAVE_SCRIPT:	return( _TL("Create Script Command File") );
 	case ID_CMD_MODULES_SAVE_DOCS:		return( _TL("Create Tool Description Files") );
+	case ID_CMD_MODULES_SAVE_SCRIPT:		return( _TL("Save to Script File") );
 	case ID_CMD_MODULES_SAVE_TO_CLIPBOARD:	return( _TL("Save to Clipboard") );
 	case ID_CMD_MODULES_CHAIN_RELOAD:	return( _TL("Reload") );
 	case ID_CMD_MODULES_CHAIN_EDIT:		return( _TL("Edit") );
@@ -145,6 +146,7 @@ wxString CMD_Get_Name(int Cmd_ID)
 	case ID_CMD_DATA_PROJECT_OPEN_ADD:	return( _TL("Add Project") );
 	case ID_CMD_DATA_PROJECT_SAVE:		return( _TL("Save Project") );
 	case ID_CMD_DATA_PROJECT_SAVE_AS:	return( _TL("Save Project As...") );
+	case ID_CMD_DATA_PROJECT_BROWSE:	return( _TL("Search for Projects...") );		
 	case ID_CMD_DATA_LEGEND_COPY:		return( _TL("Copy Legend to Clipboard...") );
 	case ID_CMD_DATA_LEGEND_SIZE_INC:	return( _TL("Increase Legend Size") );
 	case ID_CMD_DATA_LEGEND_SIZE_DEC:	return( _TL("Decrease Legend Size") );
