@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: view_table_control.cpp 2074 2014-03-31 10:32:12Z oconrad $
+ * Version $Id: view_table_control.cpp 2355 2014-12-19 09:55:41Z reklov_w $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -621,16 +621,22 @@ void CVIEW_Table_Control::On_Field_Add(wxCommandEvent &event)
 	P.Add_Choice(
 		NULL	, "TYPE"	, _TL("Field Type"),
 		_TL(""),
-		CSG_String::Format(SG_T("%s|%s|%s|%s|%s|%s|%s|%s|"),
-			_TL("character string"),
-			_TL("1 byte integer"),
-			_TL("2 byte integer"),
-			_TL("4 byte integer"),
-			_TL("4 byte floating point"),
-			_TL("8 byte floating point"),
-			_TL("date (dd.mm.yyyy)"),
-			_TL("color (rgb)")
-		)
+		CSG_String::Format(SG_T("%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|"),
+			SG_Data_Type_Get_Name(SG_DATATYPE_String).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Date  ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Color ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Byte  ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Char  ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Word  ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Short ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_DWord ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Int   ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_ULong ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Long  ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Float ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Double).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Binary).c_str()
+		), 0
 	);
 
 	P.Add_Choice(
@@ -657,14 +663,20 @@ void CVIEW_Table_Control::On_Field_Add(wxCommandEvent &event)
 		switch( P("TYPE")->asInt() )
 		{
 		default:
-		case 0:	Type	= SG_DATATYPE_String;	break;
-		case 1:	Type	= SG_DATATYPE_Char;		break;
-		case 2:	Type	= SG_DATATYPE_Short;	break;
-		case 3:	Type	= SG_DATATYPE_Int;		break;
-		case 4:	Type	= SG_DATATYPE_Float;	break;
-		case 5:	Type	= SG_DATATYPE_Double;	break;
-		case 6:	Type	= SG_DATATYPE_Date;		break;
-		case 7:	Type	= SG_DATATYPE_Color;	break;
+		case  0:	Type	= SG_DATATYPE_String;	break;
+		case  1:	Type	= SG_DATATYPE_Date;		break;
+		case  2:	Type	= SG_DATATYPE_Color;	break;
+		case  3:	Type	= SG_DATATYPE_Byte;		break;
+		case  4:	Type	= SG_DATATYPE_Char;		break;
+		case  5:	Type	= SG_DATATYPE_Word;		break;
+		case  6:	Type	= SG_DATATYPE_Short;	break;
+		case  7:	Type	= SG_DATATYPE_DWord;	break;
+		case  8:	Type	= SG_DATATYPE_Int;		break;
+		case  9:	Type	= SG_DATATYPE_ULong;	break;
+		case 10:	Type	= SG_DATATYPE_Long;		break;
+		case 11:	Type	= SG_DATATYPE_Float;	break;
+		case 12:	Type	= SG_DATATYPE_Double;	break;
+		case 13:	Type	= SG_DATATYPE_Binary;	break;
 		}
 
 		Position	= P("FIELD")->asInt() + P("INSERT")->asInt();
@@ -815,26 +827,38 @@ void CVIEW_Table_Control::On_Field_Type(wxCommandEvent &event)
 		switch( m_pTable->Get_Field_Type(i) )
 		{
 		default:
-		case SG_DATATYPE_String:	Types[i]	= 0;	break;
-		case SG_DATATYPE_Char:		Types[i]	= 1;	break;
-		case SG_DATATYPE_Short:		Types[i]	= 2;	break;
-		case SG_DATATYPE_Int:		Types[i]	= 3;	break;
-		case SG_DATATYPE_Float:	    Types[i]	= 4;	break;
-		case SG_DATATYPE_Double:	Types[i]	= 5;	break;
-		case SG_DATATYPE_Date:	    Types[i]	= 6;	break;
-		case SG_DATATYPE_Binary:	Types[i]	= 7;	break;
+		case SG_DATATYPE_String:	Types[i]	=  0;	break;
+		case SG_DATATYPE_Date:		Types[i]	=  1;	break;
+		case SG_DATATYPE_Color:		Types[i]	=  2;	break;
+		case SG_DATATYPE_Byte:		Types[i]	=  3;	break;
+		case SG_DATATYPE_Char:		Types[i]	=  4;	break;
+		case SG_DATATYPE_Word:		Types[i]	=  5;	break;
+		case SG_DATATYPE_Short:		Types[i]	=  6;	break;
+		case SG_DATATYPE_DWord:		Types[i]	=  7;	break;
+		case SG_DATATYPE_Int:		Types[i]	=  8;	break;
+		case SG_DATATYPE_ULong:		Types[i]	=  9;	break;
+		case SG_DATATYPE_Long:		Types[i]	= 10;	break;
+		case SG_DATATYPE_Float:		Types[i]	= 11;	break;
+		case SG_DATATYPE_Double:	Types[i]	= 12;	break;
+		case SG_DATATYPE_Binary:	Types[i]	= 13;	break;
 		}
 
         P.Add_Choice(NULL, "", m_pTable->Get_Field_Name(i), _TL(""),
-            CSG_String::Format(SG_T("%s|%s|%s|%s|%s|%s|%s|%s|"),
-                _TL("character string"),
-                _TL("1 byte integer"),
-                _TL("2 byte integer"),
-                _TL("4 byte integer"),
-                _TL("4 byte floating point"),
-                _TL("8 byte floating point"),
-                _TL("date (dd.mm.yyyy)"),
-                _TL("color (rgb)")
+            CSG_String::Format(SG_T("%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|"),
+			SG_Data_Type_Get_Name(SG_DATATYPE_String).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Date  ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Color ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Byte  ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Char  ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Word  ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Short ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_DWord ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Int   ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_ULong ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Long  ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Float ).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Double).c_str(),
+			SG_Data_Type_Get_Name(SG_DATATYPE_Binary).c_str()
             ), Types[i]
         );
 	}
@@ -851,14 +875,20 @@ void CVIEW_Table_Control::On_Field_Type(wxCommandEvent &event)
 			switch( P(i)->asInt() )
 			{
 			default:
-			case 0:	Type	= SG_DATATYPE_String;	break;
-            case 1:	Type	= SG_DATATYPE_Char;		break;
-            case 2:	Type	= SG_DATATYPE_Short;	break;
-            case 3:	Type	= SG_DATATYPE_Int;		break;
-            case 4:	Type	= SG_DATATYPE_Float;	break;
-            case 5:	Type	= SG_DATATYPE_Double;	break;
-            case 6:	Type	= SG_DATATYPE_Date;		break;
-            case 7:	Type	= SG_DATATYPE_Color;	break;
+			case  0:	Type	= SG_DATATYPE_String;	break;
+			case  1:	Type	= SG_DATATYPE_Date;		break;
+			case  2:	Type	= SG_DATATYPE_Color;	break;
+			case  3:	Type	= SG_DATATYPE_Byte;		break;
+			case  4:	Type	= SG_DATATYPE_Char;		break;
+			case  5:	Type	= SG_DATATYPE_Word;		break;
+			case  6:	Type	= SG_DATATYPE_Short;	break;
+			case  7:	Type	= SG_DATATYPE_DWord;	break;
+			case  8:	Type	= SG_DATATYPE_Int;		break;
+			case  9:	Type	= SG_DATATYPE_ULong;	break;
+			case 10:	Type	= SG_DATATYPE_Long;		break;
+			case 11:	Type	= SG_DATATYPE_Float;	break;
+			case 12:	Type	= SG_DATATYPE_Double;	break;
+			case 13:	Type	= SG_DATATYPE_Binary;	break;
 			}
 
 			if( Type != Types[i] )
@@ -872,6 +902,7 @@ void CVIEW_Table_Control::On_Field_Type(wxCommandEvent &event)
 		if( bChanged )
 		{
 			Update_Table();
+			g_pData->Update(m_pTable, NULL);
 		}
 	}
 }
