@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_map.cpp 2331 2014-11-07 12:15:10Z oconrad $
+ * Version $Id: wksp_map.cpp 2495 2015-05-21 14:14:52Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -455,6 +455,13 @@ void CWKSP_Map::On_Create_Parameters(void)
 		PARAMETER_TYPE_Double, 90, 0, true, 100, true
 	);
 
+	//-----------------------------------------------------
+	pNode_1	= m_Parameters.Add_Value(
+		pNode_0	, "GCS_POSITION"	, _TL("Position as Geographic Coordinates"),
+		_TL("show mouse position in statusbar using geographic coordinates if possible"),
+		PARAMETER_TYPE_Bool, false
+	);
+
 	//-----------------------------------------------------
 	pNode_1	= m_Parameters.Add_Value(
 		pNode_0	, "SCALE_SHOW"		, _TL("Scale Bar"),
@@ -1710,14 +1717,25 @@ void CWKSP_Map::Draw_Frame(wxDC &dc, const CSG_Rect &rWorld, wxRect rMap, int Wi
 	r		= wxRect(rMap.GetLeft()    , rFrame.GetTop()    , rMap.GetWidth(), Width);
 	Draw_Scale(dc, r, rWorld.Get_XMin(), rWorld.Get_XMax()  , true , true , false);
 
-	r		= wxRect(rMap.GetLeft()    , rMap.GetBottom()   , rMap.GetWidth(), Width);
-	Draw_Scale(dc, r, 0.0              , rWorld.Get_XRange(), true , true , true);
-
 	r		= wxRect(rFrame.GetLeft()  , rMap.GetTop()      , Width, rMap.GetHeight());
 	Draw_Scale(dc, r, rWorld.Get_YMin(), rWorld.Get_YMax()  , false, false, false);
 
-	r		= wxRect(rMap.GetRight()   , rMap.GetTop()      , Width, rMap.GetHeight());
-	Draw_Scale(dc, r, 0.0              , rWorld.Get_YRange(), false, false, true);
+	if( is_ScaleBar() )
+	{
+		r		= wxRect(rMap.GetLeft()    , rMap.GetBottom()   , rMap.GetWidth(), Width);
+		Draw_Scale(dc, r, rWorld.Get_XMin(), rWorld.Get_XMax()  , true , true , false);
+
+		r		= wxRect(rMap.GetRight()   , rMap.GetTop()      , Width, rMap.GetHeight());
+		Draw_Scale(dc, r, rWorld.Get_YMin(), rWorld.Get_YMax()  , false, false, false);
+	}
+	else
+	{
+		r		= wxRect(rMap.GetLeft()    , rMap.GetBottom()   , rMap.GetWidth(), Width);
+		Draw_Scale(dc, r, 0.0              , rWorld.Get_XRange(), true , true , true);
+
+		r		= wxRect(rMap.GetRight()   , rMap.GetTop()      , Width, rMap.GetHeight());
+		Draw_Scale(dc, r, 0.0              , rWorld.Get_YRange(), false, false, true);
+	}
 
 	Draw_Edge(dc, EDGE_STYLE_SIMPLE, rFrame.GetLeft(), rFrame.GetTop(), rFrame.GetRight(), rFrame.GetBottom());
 }
