@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_module_manager.cpp 2281 2014-10-09 15:49:41Z oconrad $
+ * Version $Id: wksp_module_manager.cpp 2496 2015-05-21 14:16:21Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -162,13 +162,13 @@ CWKSP_Module_Manager::CWKSP_Module_Manager(void)
 
 	m_Parameters.Add_Value(
 		pNode	, "LNG_OLDSTYLE"	, _TL("Old Style Namings"),
-		_TL("Use old style namings (e.g. 'modules' instead of 'tools'). Ignored if translation file is used. You need to restart SAGA to apply changes."),
+		_TL("Use old style namings (e.g. 'modules' instead of 'tools'). Ignored if translation file is used. You need to restart SAGA to apply the changes."),
 		PARAMETER_TYPE_Bool, false
 	);
 
 	m_Parameters.Add_FilePath(
 		pNode	, "LNG_FILE_DIC"	, _TL("Language Translations"),
-		_TL("Dictionary for translations from built-in (English) to local language (editable text table). You need to restart SAGA to apply changes."),
+		_TL("Dictionary for translations from built-in (English) to local language (editable text table). You need to restart SAGA to apply the changes."),
 		CSG_String::Format(SG_T("%s|*.lng|%s|*.txt|%s|*.*"),
 			_TL("Dictionary Files (*.lng)"),
 			_TL("Text Table (*.txt)"),
@@ -178,7 +178,7 @@ CWKSP_Module_Manager::CWKSP_Module_Manager(void)
 
 	m_Parameters.Add_FilePath(
 		pNode	, "CRS_FILE_SRS"		, _TL("CRS Database"),
-		_TL("Database with Coordinate Reference System (CRS) definitions. You have to restart SAGA to make changes take affect!"),
+		_TL("Database with Coordinate Reference System (CRS) definitions. A restart of SAGA is required to have the changes take effect!"),
 		CSG_String::Format(SG_T("%s|*.srs|%s|*.*"),
 			_TL("Spatial Reference System Files (*.srs)"),
 			_TL("All Files")
@@ -187,12 +187,21 @@ CWKSP_Module_Manager::CWKSP_Module_Manager(void)
 
 	m_Parameters.Add_FilePath(
 		pNode	, "CRS_FILE_DIC"		, _TL("CRS Dictionary"),
-		_TL("Dictionary for Proj.4/OGC WKT translations. You have to restart SAGA to make changes take affect!"),
+		_TL("Dictionary for Proj.4/OGC WKT translations. A restart of SAGA is required to have the changes take effect!"),
 		CSG_String::Format(SG_T("%s|*.dic|%s|*.*"),
 			_TL("Dictionary Files (*.dic)"),
 			_TL("All Files")
 		)
 	);
+
+	m_Parameters.Add_FilePath(
+		pNode	, "TOOL_MENUS"			, _TL("User defined tool menus"),
+		_TL("User defined tool menus."),
+		CSG_String::Format(SG_T("%s|*.xml|%s|*.*"),
+			_TL("XML Files (*.xml)"),
+			_TL("All Files")
+		)
+	);
 }
 
 //---------------------------------------------------------
@@ -230,6 +239,13 @@ bool CWKSP_Module_Manager::Initialise(void)
 
 	for(int i=0; CONFIG_Read(CFG_LIBS, wxString::Format(CFG_LIBF, i), Library); i++)
 	{
+		if( !wxFileExists(Library) )
+		{
+			wxFileName	fn(Library);	fn.MakeAbsolute(g_pSAGA->Get_App_Path());
+
+			Library	= fn.GetFullPath();
+		}
+
 		SG_Get_Module_Library_Manager().Add_Library(Library);
 	}
 
@@ -444,6 +460,8 @@ void CWKSP_Module_Manager::Parameters_Changed(void)
 #ifdef _OPENMP
 	SG_Set_Max_Num_Threads_Omp(m_Parameters("OMP_THREADS_MAX")->asInt());
 #endif
+
+	m_pMenu_Modules->Update();
 }
 
 //---------------------------------------------------------
@@ -493,20 +511,16 @@ CWKSP_Module_Library * CWKSP_Module_Manager::Get_Library(CSG_Module_Library *pLi
 //---------------------------------------------------------
 bool CWKSP_Module_Manager::_Update(bool bSyncToCtrl)
 {
-//	Get_Control()->Freeze();
-
-	for(int i=SG_Get_Module_Library_Manager().Get_Count()-1; i>=0; i--)
+	if( !bSyncToCtrl )
 	{
-		CSG_Module_Library	*pLibrary	= SG_Get_Module_Library_Manager().Get_Library(i);
-		CWKSP_Module_Library	*pItem	= Get_Library(pLibrary);
+	//	Get_Control()->Freeze();
 
-		if( !pItem )
+		for(int i=SG_Get_Module_Library_Manager().Get_Count()-1; i>=0; i--)
 		{
-			if( bSyncToCtrl )
-			{
-				SG_Get_Module_Library_Manager().Del_Library(i);
-			}
-			else
+			CSG_Module_Library	*pLibrary	= SG_Get_Module_Library_Manager().Get_Library(i);
+			CWKSP_Module_Library	*pItem	= Get_Library(pLibrary);
+
+			if( !pItem )
 			{
 				CWKSP_Module_Group	*pGroup	= Get_Group(pLibrary->Get_Category().c_str());
 
@@ -517,14 +531,14 @@ bool CWKSP_Module_Manager::_Update(bool bSyncToCtrl)
 
 				pGroup->Add_Library(pLibrary);
 			}
+			else if( pItem->Get_Library()->Get_Type() == MODULE_CHAINS )
+			{
+				pItem->Update();
+			}
 		}
-		else if( !bSyncToCtrl && pItem->Get_Library()->Get_Type() == MODULE_CHAINS )
-		{
-			pItem->Update();
-		}
-	}
 
-//	Get_Control()->Thaw();
+	//	Get_Control()->Thaw();
+	}
 
 	m_pMenu_Modules->Update();
 
